@@ -24,10 +24,12 @@ use NetAddr::IP::Util qw(
 	ipv6_n2x
 	mask4to6
 	ipv4to6
+	naip_gethostbyname
 );
+
 use vars qw(@ISA @EXPORT_OK $VERSION $Accept_Binary_IP $Old_nth $AUTOLOAD *Zero);
 
-$VERSION = do { my @r = (q$Revision: 1.13 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 1.24 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 require Exporter;
 
@@ -56,6 +58,8 @@ NetAddr::IP::Lite - Manages IPv4 and IPv6 addresses and subnets
 	V4net
 	:aton		DEPRECATED !
 	:old_nth
+	:upper
+	:lower
   );
 
   my $ip = new NetAddr::IP::Lite '127.0.0.1';
@@ -106,6 +110,26 @@ addresses or IP subnets, that allows for easy manipulations. Most of the
 operations of NetAddr::IP are supported. This module will work older
 versions of Perl and does B<not> use Math::BigInt.
 
+* By default B<NetAddr::IP> functions and methods return string IPv6
+addresses in uppercase.  To change that to lowercase:
+
+NOTE: the AUGUST 2010 RFC5952 states:
+
+    4.3. Lowercase
+
+      The characters "a", "b", "c", "d", "e", and "f" in an IPv6
+      address MUST be represented in lowercase.
+
+It is recommended that all NEW applications using NetAddr::IP::Lite be
+invoked as shown on the next line.
+
+  use NetAddr::IP::Lite qw(:lower);
+
+* To ensure the current IPv6 string case behavior even if the default changes:
+
+  use NetAddr::IP::Lite qw(:upper);
+
+
 The internal representation of all IP objects is in 128 bit IPv6 notation.
 IPv4 and IPv6 objects may be freely mixed.
 
@@ -205,11 +229,22 @@ use overload
 	$a eq $b;
     },
 
+    'ne'	=> sub {
+	my $a = (UNIVERSAL::isa($_[0],__PACKAGE__)) ? $_[0]->cidr : $_[0];
+	my $b = (UNIVERSAL::isa($_[1],__PACKAGE__)) ? $_[1]->cidr : $_[1];
+	$a ne $b;
+    },
+
     '=='	=> sub {
 	return 0 unless UNIVERSAL::isa($_[0],__PACKAGE__) && UNIVERSAL::isa($_[1],__PACKAGE__);
 	$_[0]->cidr eq $_[1]->cidr;
     },
 
+    '!='	=> sub {
+	return 1 unless UNIVERSAL::isa($_[0],__PACKAGE__) && UNIVERSAL::isa($_[1],__PACKAGE__);
+	$_[0]->cidr ne $_[1]->cidr;
+    },
+
     '>'		=> sub {
 	return &comp_addr_mask > 0 ? 1 : 0;
     },
@@ -279,11 +314,11 @@ Will print the string 192.168.1.123/32.
 	my $ip = new6 NetAddr::IP::Lite '192.168.1.123';
 	print "$ip\n";
 
-Will print the string
+Will print the string 0:0:0:0:0:0:C0A8:17B/128
 
 =item B<Equality>
 
-You can test for equality with either C<eq> or C<==>. C<eq> allows the
+You can test for equality with either C<eq>, C<ne>, C<==> or C<!=>. C<eq>, C<ne> allows the
 comparison with arbitrary strings as well as NetAddr::IP::Lite objects. The
 following example:
 
@@ -292,10 +327,7 @@ following example:
 
 Will print out "Yes".
 
-Comparison with C<==> requires both operands to be NetAddr::IP::Lite objects.
-
-In both cases, a true value is returned if the CIDR representation of
-the operands is equal.
+Comparison with C<==> and C<!=> requires both operands to be NetAddr::IP::Lite objects.
 
 =item B<Comparison via E<gt>, E<lt>, E<gt>=, E<lt>=, E<lt>=E<gt> and C<cmp>>
 
@@ -471,8 +503,14 @@ sub _new ($$$) {
 
 =item C<-E<gt>new_from_aton($netaddr)>
 
+=item new_cis and new_cis6 are DEPRECATED
+
+=item C<-E<gt>new_cis("$addr $mask)>
+
+=item C<-E<gt>new_cis6("$addr $mask)>
+
 The first two methods create a new address with the supplied address in
-C<$addr> and an optional netmask C<$mask>, which can be omitted to get
+C<$addr> and an optional netmask C<$mask>, which can be omitted to get 
 a /32 or /128 netmask for IPv4 / IPv6 addresses respectively.
 
 The third method C<new_no> is exclusively for IPv4 addresses and filters
@@ -484,21 +522,35 @@ B<new_from_aton> takes a packed IPv4 address and assumes a /32 mask. This
 function replaces the DEPRECATED :aton functionality which is fundamentally
 broken.
 
-C<-E<gt>new6> marks the address as being in ipV6 address space even if the
-format would suggest otherwise.
+The last two methods B<new_cis> and B<new_cis6> differ from B<new> and
+B<new6> only in that they except the common Cisco address notation for
+address/mask pairs with a B<space> as a separator instead of a slash (/)
 
-  i.e.	->new6('1.2.3.4') will result in ::102:304
+These methods are DEPRECATED because the functionality is now included
+in the other "new" methods
+
+  i.e.  ->new_cis('1.2.3.0 24')
+        or
+        ->new_cis6('::1.2.3.0 120')
+
+C<-E<gt>new6> and
+C<-E<gt>new_cis6> mark the address as being in ipV6 address space even
+if the format would suggest otherwise.
+
+  i.e.  ->new6('1.2.3.4') will result in ::102:304
 
   addresses submitted to ->new in ipV6 notation will
   remain in that notation permanently. i.e.
-	->new('::1.2.3.4') will result in ::102:304
+        ->new('::1.2.3.4') will result in ::102:304
   whereas new('1.2.3.4') would print out as 1.2.3.4
 
   See "STRINGIFICATION" below.
 
 C<$addr> can be almost anything that can be resolved to an IP address
 in all the notations I have seen over time. It can optionally contain
-the mask in CIDR notation.
+the mask in CIDR notation. If the OPTIONAL perl module Socket6 is
+available in the local library it will autoload and ipV6 host6 
+names will be resolved as well as ipV4 hostnames.
 
 B<prefix> notation is understood, with the limitation that the range
 specified by the prefix must match with a valid subnet.
@@ -521,11 +573,15 @@ C<$addr> can be any of the following and possibly more...
 
   n.n
   n.n/mm
+  n.n mm
   n.n.n
   n.n.n/mm
+  n.n.n mm
   n.n.n.n
   n.n.n.n/mm		32 bit cidr notation
+  n.n.n.n mm
   n.n.n.n/m.m.m.m
+  n.n.n.n m.m.m.m
   loopback, localhost, broadcast, any, default
   x.x.x.x/host
   0xABCDEF, 0b111111000101011110, (or a bcd number)
@@ -626,6 +682,24 @@ sub new6($;$$) {
   goto &_xnew;
 }
 
+sub new_cis($;$$) {
+  my @in = @_;
+  if ( $in[1] && $in[1] =~ m!^(.+)\s+(.+)$! ) {
+    $in[1] = $1 .'/'. $2;
+  }
+  @_ = (0,@in);
+  goto &_xnew;
+}
+
+sub new_cis6($;$$) {
+  my @in = @_;
+  if ( $in[1] && $in[1] =~ m!^(.+)\s+(.+)$! ) {
+    $in[1] = $1 .'/'. $2;
+  }
+  @_ = (1,@in);
+  goto &_xnew;
+}
+
 sub _no_octal {
   $_[0] =~ m/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
   return sprintf("%d.%d.%d.%d",$1,$2,$3,$4);
@@ -647,7 +721,8 @@ sub _xnew($$;$$) {
 
   while (1) {
     unless (@_) {
-      if ($ip =~ m!^(.+)/(.+)$!) {
+#      if ($ip =~ m!^(.+)/(.+)$!) {
+      if ($ip =~ m!^([a-z0-9.:-]+)(?:/|\s+)([a-z0-9.:-]+)$!) {
 	$ip	= $1;
 	$mask	= $2;
       } elsif (grep($ip eq $_,qw(default any broadcast loopback unspecified))) {
@@ -669,7 +744,7 @@ sub _xnew($$;$$) {
 	  $ip	= $fip6{$ip};
 	  last;
 	} else {
-	  return undef;
+	  return undef unless $isV6;
         }
       } else {
 	$mask = lc $_[0];
@@ -682,24 +757,28 @@ sub _xnew($$;$$) {
 
 # parse mask
     if ($mask =~ /^(\d+)$/) {
-      if (index($ip,':') < 0) {			# is ipV4
-	if ($1 == 32) {				# cidr 32
+      my $mval = $1;
+      if (! $isV6 && index($ip,':') < 0) {	# is ipV4
+	if ($mval == 32) {				# cidr 32
 	  $mask = Ones;
 	}
 	elsif ($mask < 32) {			# small cidr
-	  $mask = shiftleft(Ones,32 -$1);
+	  $mask = shiftleft(Ones,32 -$mval);
 	} else {				# is a binary mask
-	  $mask = pack('L3N',0xffffffff,0xffffffff,0xffffffff,$1);
+	  $mask = pack('L3N',0xffffffff,0xffffffff,0xffffffff,$mval);
 	}
       } else {					# is ipV6
 	$isV6	= 1;
-	if ($1 == 128) {			# cidr 128
+	if ($mval == 128) {			# cidr 128
 	  $mask = Ones;
 	}
+	elsif (index($ip,':') < 0) {	# corner case of ipV4 with new6
+	  $mask = shiftleft(Ones,32 -$mval);
+	}
 	elsif ($mask < 128) {			# small cidr
-	  $mask = shiftleft(Ones,128 -$1);
+	  $mask = shiftleft(Ones,128 -$mval);
 	} else {				# is a binary mask
-	  $mask = bcd2bin($1);
+	  $mask = bcd2bin($mval);
 	}
       }
     } elsif ($mask =~ m/^\d+\.\d+\.\d+\.\d+$/) { # ipv4 form of mask
@@ -809,10 +888,17 @@ sub _xnew($$;$$) {
 	return undef if hasbits($ip & $tmp);
 	last;
       }
+# check for resolvable IPv4 hosts
       elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && ($tmp = gethostbyname($ip)) && $tmp ne $_v4zero && $tmp ne $_zero ) {
 	$ip = ipv4to6($tmp);
 	last;
       }
+# check for resolvable IPv6 hosts
+      elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && ($tmp = naip_gethostbyname($ip))) {
+	$ip = $tmp;
+	$isV6 = 1;
+	last;
+      }
       elsif ($Accept_Binary_IP && ! $hasmask) {
 	if (length($ip) == 4) {
 	  $ip = ipv4to6($ip);
@@ -1164,6 +1250,16 @@ sub import {
     $Old_nth = 1;
     @_ = grep { $_ ne ':old_nth' } @_;
   }
+  if (grep { $_ eq ':lower' } @_)
+  {
+    NetAddr::IP::Util::lower();
+    @_ = grep { $_ ne ':lower' } @_;
+  }
+  if (grep { $_ eq ':upper' } @_)
+  {
+    NetAddr::IP::Util::upper();
+    @_ = grep { $_ ne ':upper' } @_;
+  }
   NetAddr::IP::Lite->export_to_level(1, @_);
 }
 
@@ -1175,6 +1271,8 @@ sub import {
 	V4net
 	:aton		DEPRECATED
 	:old_nth
+	:upper
+	:lower
 
 =head1 AUTHOR
 
@@ -1189,7 +1287,7 @@ so by using it you accept any and all the liability.
 =head1 LICENSE
 
  This software is (c) Luis E. Muoz, 1999 - 2005
- and (c) Michael Robinton, 2006 - 2008.
+ and (c) Michael Robinton, 2006 - 2010.
 
 It can be used under the terms of the perl artistic license provided that
 proper credit for the work of the author is preserved in the form of this
